from enum import Enum
import math
import string


class VitaminAUnits(Enum):
        """
            VitaminAUnits enumeration
        """
        
        InternationalUnit = 'international_unit'
        """
            
        """
        

class VitaminA:
    """
    Vitamin A: 1 IU is the biological equivalent of 0.3 µg retinol, or of 0.6 µg beta-carotene.

    Args:
        value (float): The value.
        from_unit (VitaminAUnits): The VitaminA unit to create from, The default unit is InternationalUnit
    """
    def __init__(self, value: float, from_unit: VitaminAUnits = VitaminAUnits.InternationalUnit):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__international_units = None
        

    def __convert_from_base(self, from_unit: VitaminAUnits) -> float:
        value = self.__value
        
        if from_unit == VitaminAUnits.InternationalUnit:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: VitaminAUnits) -> float:
        
        if to_unit == VitaminAUnits.InternationalUnit:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_international_units(international_units: float):
        """
        Create a new instance of VitaminA from a value in international_units.

        

        :param meters: The VitaminA value in international_units.
        :type international_units: float
        :return: A new instance of VitaminA.
        :rtype: VitaminA
        """
        return VitaminA(international_units, VitaminAUnits.InternationalUnit)

    
    @property
    def international_units(self) -> float:
        """
        
        """
        if self.__international_units != None:
            return self.__international_units
        self.__international_units = self.__convert_from_base(VitaminAUnits.InternationalUnit)
        return self.__international_units

    
    def to_string(self, unit: VitaminAUnits = VitaminAUnits.InternationalUnit) -> string:
        """
        Format the VitaminA to string.
        Note! the default format for VitaminA is InternationalUnit.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == VitaminAUnits.InternationalUnit:
            return f"""{self.international_units} IU"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: VitaminAUnits = VitaminAUnits.InternationalUnit) -> string:
        """
        Get VitaminA unit abbreviation.
        Note! the default abbreviation for VitaminA is InternationalUnit.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VitaminAUnits.InternationalUnit:
            return """IU"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for +: 'VitaminA' and '{}'".format(type(other).__name__))
        return VitaminA(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for *: 'VitaminA' and '{}'".format(type(other).__name__))
        return VitaminA(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for -: 'VitaminA' and '{}'".format(type(other).__name__))
        return VitaminA(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for /: 'VitaminA' and '{}'".format(type(other).__name__))
        return VitaminA(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for %: 'VitaminA' and '{}'".format(type(other).__name__))
        return VitaminA(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for **: 'VitaminA' and '{}'".format(type(other).__name__))
        return VitaminA(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for ==: 'VitaminA' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for <: 'VitaminA' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for >: 'VitaminA' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for <=: 'VitaminA' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, VitaminA):
            raise TypeError("unsupported operand type(s) for >=: 'VitaminA' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ElectricCurrentDensityUnits(Enum):
        """
            ElectricCurrentDensityUnits enumeration
        """
        
        AmperePerSquareMeter = 'ampere_per_square_meter'
        """
            
        """
        
        AmperePerSquareInch = 'ampere_per_square_inch'
        """
            
        """
        
        AmperePerSquareFoot = 'ampere_per_square_foot'
        """
            
        """
        

class ElectricCurrentDensity:
    """
    In electromagnetism, current density is the electric current per unit area of cross section.

    Args:
        value (float): The value.
        from_unit (ElectricCurrentDensityUnits): The ElectricCurrentDensity unit to create from, The default unit is AmperePerSquareMeter
    """
    def __init__(self, value: float, from_unit: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__amperes_per_square_meter = None
        
        self.__amperes_per_square_inch = None
        
        self.__amperes_per_square_foot = None
        

    def __convert_from_base(self, from_unit: ElectricCurrentDensityUnits) -> float:
        value = self.__value
        
        if from_unit == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return (value)
        
        if from_unit == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return (value / 1.5500031000062000e3)
        
        if from_unit == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return (value / 1.0763910416709722e1)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricCurrentDensityUnits) -> float:
        
        if to_unit == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return (value)
        
        if to_unit == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return (value * 1.5500031000062000e3)
        
        if to_unit == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return (value * 1.0763910416709722e1)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_amperes_per_square_meter(amperes_per_square_meter: float):
        """
        Create a new instance of ElectricCurrentDensity from a value in amperes_per_square_meter.

        

        :param meters: The ElectricCurrentDensity value in amperes_per_square_meter.
        :type amperes_per_square_meter: float
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity(amperes_per_square_meter, ElectricCurrentDensityUnits.AmperePerSquareMeter)

    
    @staticmethod
    def from_amperes_per_square_inch(amperes_per_square_inch: float):
        """
        Create a new instance of ElectricCurrentDensity from a value in amperes_per_square_inch.

        

        :param meters: The ElectricCurrentDensity value in amperes_per_square_inch.
        :type amperes_per_square_inch: float
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity(amperes_per_square_inch, ElectricCurrentDensityUnits.AmperePerSquareInch)

    
    @staticmethod
    def from_amperes_per_square_foot(amperes_per_square_foot: float):
        """
        Create a new instance of ElectricCurrentDensity from a value in amperes_per_square_foot.

        

        :param meters: The ElectricCurrentDensity value in amperes_per_square_foot.
        :type amperes_per_square_foot: float
        :return: A new instance of ElectricCurrentDensity.
        :rtype: ElectricCurrentDensity
        """
        return ElectricCurrentDensity(amperes_per_square_foot, ElectricCurrentDensityUnits.AmperePerSquareFoot)

    
    @property
    def amperes_per_square_meter(self) -> float:
        """
        
        """
        if self.__amperes_per_square_meter != None:
            return self.__amperes_per_square_meter
        self.__amperes_per_square_meter = self.__convert_from_base(ElectricCurrentDensityUnits.AmperePerSquareMeter)
        return self.__amperes_per_square_meter

    
    @property
    def amperes_per_square_inch(self) -> float:
        """
        
        """
        if self.__amperes_per_square_inch != None:
            return self.__amperes_per_square_inch
        self.__amperes_per_square_inch = self.__convert_from_base(ElectricCurrentDensityUnits.AmperePerSquareInch)
        return self.__amperes_per_square_inch

    
    @property
    def amperes_per_square_foot(self) -> float:
        """
        
        """
        if self.__amperes_per_square_foot != None:
            return self.__amperes_per_square_foot
        self.__amperes_per_square_foot = self.__convert_from_base(ElectricCurrentDensityUnits.AmperePerSquareFoot)
        return self.__amperes_per_square_foot

    
    def to_string(self, unit: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter) -> string:
        """
        Format the ElectricCurrentDensity to string.
        Note! the default format for ElectricCurrentDensity is AmperePerSquareMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return f"""{self.amperes_per_square_meter} A/m²"""
        
        if unit == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return f"""{self.amperes_per_square_inch} A/in²"""
        
        if unit == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return f"""{self.amperes_per_square_foot} A/ft²"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricCurrentDensityUnits = ElectricCurrentDensityUnits.AmperePerSquareMeter) -> string:
        """
        Get ElectricCurrentDensity unit abbreviation.
        Note! the default abbreviation for ElectricCurrentDensity is AmperePerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricCurrentDensityUnits.AmperePerSquareMeter:
            return """A/m²"""
        
        if unit_abbreviation == ElectricCurrentDensityUnits.AmperePerSquareInch:
            return """A/in²"""
        
        if unit_abbreviation == ElectricCurrentDensityUnits.AmperePerSquareFoot:
            return """A/ft²"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for +: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return ElectricCurrentDensity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for *: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return ElectricCurrentDensity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for -: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return ElectricCurrentDensity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for /: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return ElectricCurrentDensity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for %: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return ElectricCurrentDensity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for **: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return ElectricCurrentDensity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for ==: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for <: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for >: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for <=: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ElectricCurrentDensity):
            raise TypeError("unsupported operand type(s) for >=: 'ElectricCurrentDensity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class CompressibilityUnits(Enum):
        """
            CompressibilityUnits enumeration
        """
        
        InversePascal = 'inverse_pascal'
        """
            
        """
        
        InverseKilopascal = 'inverse_kilopascal'
        """
            
        """
        
        InverseMegapascal = 'inverse_megapascal'
        """
            
        """
        
        InverseAtmosphere = 'inverse_atmosphere'
        """
            
        """
        
        InverseMillibar = 'inverse_millibar'
        """
            
        """
        
        InverseBar = 'inverse_bar'
        """
            
        """
        
        InversePoundForcePerSquareInch = 'inverse_pound_force_per_square_inch'
        """
            
        """
        

class Compressibility:
    """
    None

    Args:
        value (float): The value.
        from_unit (CompressibilityUnits): The Compressibility unit to create from, The default unit is InversePascal
    """
    def __init__(self, value: float, from_unit: CompressibilityUnits = CompressibilityUnits.InversePascal):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__inverse_pascals = None
        
        self.__inverse_kilopascals = None
        
        self.__inverse_megapascals = None
        
        self.__inverse_atmospheres = None
        
        self.__inverse_millibars = None
        
        self.__inverse_bars = None
        
        self.__inverse_pounds_force_per_square_inch = None
        

    def __convert_from_base(self, from_unit: CompressibilityUnits) -> float:
        value = self.__value
        
        if from_unit == CompressibilityUnits.InversePascal:
            return (value)
        
        if from_unit == CompressibilityUnits.InverseKilopascal:
            return (value / 1e3)
        
        if from_unit == CompressibilityUnits.InverseMegapascal:
            return (value / 1e6)
        
        if from_unit == CompressibilityUnits.InverseAtmosphere:
            return (value / 101325)
        
        if from_unit == CompressibilityUnits.InverseMillibar:
            return (value / 100)
        
        if from_unit == CompressibilityUnits.InverseBar:
            return (value / 1e5)
        
        if from_unit == CompressibilityUnits.InversePoundForcePerSquareInch:
            return (value / 6.894757293168361e3)
        
        return None


    def __convert_to_base(self, value: float, to_unit: CompressibilityUnits) -> float:
        
        if to_unit == CompressibilityUnits.InversePascal:
            return (value)
        
        if to_unit == CompressibilityUnits.InverseKilopascal:
            return (value * 1e3)
        
        if to_unit == CompressibilityUnits.InverseMegapascal:
            return (value * 1e6)
        
        if to_unit == CompressibilityUnits.InverseAtmosphere:
            return (value * 101325)
        
        if to_unit == CompressibilityUnits.InverseMillibar:
            return (value * 100)
        
        if to_unit == CompressibilityUnits.InverseBar:
            return (value * 1e5)
        
        if to_unit == CompressibilityUnits.InversePoundForcePerSquareInch:
            return (value * 6.894757293168361e3)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_inverse_pascals(inverse_pascals: float):
        """
        Create a new instance of Compressibility from a value in inverse_pascals.

        

        :param meters: The Compressibility value in inverse_pascals.
        :type inverse_pascals: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_pascals, CompressibilityUnits.InversePascal)

    
    @staticmethod
    def from_inverse_kilopascals(inverse_kilopascals: float):
        """
        Create a new instance of Compressibility from a value in inverse_kilopascals.

        

        :param meters: The Compressibility value in inverse_kilopascals.
        :type inverse_kilopascals: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_kilopascals, CompressibilityUnits.InverseKilopascal)

    
    @staticmethod
    def from_inverse_megapascals(inverse_megapascals: float):
        """
        Create a new instance of Compressibility from a value in inverse_megapascals.

        

        :param meters: The Compressibility value in inverse_megapascals.
        :type inverse_megapascals: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_megapascals, CompressibilityUnits.InverseMegapascal)

    
    @staticmethod
    def from_inverse_atmospheres(inverse_atmospheres: float):
        """
        Create a new instance of Compressibility from a value in inverse_atmospheres.

        

        :param meters: The Compressibility value in inverse_atmospheres.
        :type inverse_atmospheres: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_atmospheres, CompressibilityUnits.InverseAtmosphere)

    
    @staticmethod
    def from_inverse_millibars(inverse_millibars: float):
        """
        Create a new instance of Compressibility from a value in inverse_millibars.

        

        :param meters: The Compressibility value in inverse_millibars.
        :type inverse_millibars: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_millibars, CompressibilityUnits.InverseMillibar)

    
    @staticmethod
    def from_inverse_bars(inverse_bars: float):
        """
        Create a new instance of Compressibility from a value in inverse_bars.

        

        :param meters: The Compressibility value in inverse_bars.
        :type inverse_bars: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_bars, CompressibilityUnits.InverseBar)

    
    @staticmethod
    def from_inverse_pounds_force_per_square_inch(inverse_pounds_force_per_square_inch: float):
        """
        Create a new instance of Compressibility from a value in inverse_pounds_force_per_square_inch.

        

        :param meters: The Compressibility value in inverse_pounds_force_per_square_inch.
        :type inverse_pounds_force_per_square_inch: float
        :return: A new instance of Compressibility.
        :rtype: Compressibility
        """
        return Compressibility(inverse_pounds_force_per_square_inch, CompressibilityUnits.InversePoundForcePerSquareInch)

    
    @property
    def inverse_pascals(self) -> float:
        """
        
        """
        if self.__inverse_pascals != None:
            return self.__inverse_pascals
        self.__inverse_pascals = self.__convert_from_base(CompressibilityUnits.InversePascal)
        return self.__inverse_pascals

    
    @property
    def inverse_kilopascals(self) -> float:
        """
        
        """
        if self.__inverse_kilopascals != None:
            return self.__inverse_kilopascals
        self.__inverse_kilopascals = self.__convert_from_base(CompressibilityUnits.InverseKilopascal)
        return self.__inverse_kilopascals

    
    @property
    def inverse_megapascals(self) -> float:
        """
        
        """
        if self.__inverse_megapascals != None:
            return self.__inverse_megapascals
        self.__inverse_megapascals = self.__convert_from_base(CompressibilityUnits.InverseMegapascal)
        return self.__inverse_megapascals

    
    @property
    def inverse_atmospheres(self) -> float:
        """
        
        """
        if self.__inverse_atmospheres != None:
            return self.__inverse_atmospheres
        self.__inverse_atmospheres = self.__convert_from_base(CompressibilityUnits.InverseAtmosphere)
        return self.__inverse_atmospheres

    
    @property
    def inverse_millibars(self) -> float:
        """
        
        """
        if self.__inverse_millibars != None:
            return self.__inverse_millibars
        self.__inverse_millibars = self.__convert_from_base(CompressibilityUnits.InverseMillibar)
        return self.__inverse_millibars

    
    @property
    def inverse_bars(self) -> float:
        """
        
        """
        if self.__inverse_bars != None:
            return self.__inverse_bars
        self.__inverse_bars = self.__convert_from_base(CompressibilityUnits.InverseBar)
        return self.__inverse_bars

    
    @property
    def inverse_pounds_force_per_square_inch(self) -> float:
        """
        
        """
        if self.__inverse_pounds_force_per_square_inch != None:
            return self.__inverse_pounds_force_per_square_inch
        self.__inverse_pounds_force_per_square_inch = self.__convert_from_base(CompressibilityUnits.InversePoundForcePerSquareInch)
        return self.__inverse_pounds_force_per_square_inch

    
    def to_string(self, unit: CompressibilityUnits = CompressibilityUnits.InversePascal) -> string:
        """
        Format the Compressibility to string.
        Note! the default format for Compressibility is InversePascal.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == CompressibilityUnits.InversePascal:
            return f"""{self.inverse_pascals} Pa⁻¹"""
        
        if unit == CompressibilityUnits.InverseKilopascal:
            return f"""{self.inverse_kilopascals} kPa⁻¹"""
        
        if unit == CompressibilityUnits.InverseMegapascal:
            return f"""{self.inverse_megapascals} MPa⁻¹"""
        
        if unit == CompressibilityUnits.InverseAtmosphere:
            return f"""{self.inverse_atmospheres} atm⁻¹"""
        
        if unit == CompressibilityUnits.InverseMillibar:
            return f"""{self.inverse_millibars} mbar⁻¹"""
        
        if unit == CompressibilityUnits.InverseBar:
            return f"""{self.inverse_bars} bar⁻¹"""
        
        if unit == CompressibilityUnits.InversePoundForcePerSquareInch:
            return f"""{self.inverse_pounds_force_per_square_inch} psi⁻¹"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: CompressibilityUnits = CompressibilityUnits.InversePascal) -> string:
        """
        Get Compressibility unit abbreviation.
        Note! the default abbreviation for Compressibility is InversePascal.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == CompressibilityUnits.InversePascal:
            return """Pa⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseKilopascal:
            return """kPa⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseMegapascal:
            return """MPa⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseAtmosphere:
            return """atm⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseMillibar:
            return """mbar⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InverseBar:
            return """bar⁻¹"""
        
        if unit_abbreviation == CompressibilityUnits.InversePoundForcePerSquareInch:
            return """psi⁻¹"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for +: 'Compressibility' and '{}'".format(type(other).__name__))
        return Compressibility(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for *: 'Compressibility' and '{}'".format(type(other).__name__))
        return Compressibility(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for -: 'Compressibility' and '{}'".format(type(other).__name__))
        return Compressibility(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for /: 'Compressibility' and '{}'".format(type(other).__name__))
        return Compressibility(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for %: 'Compressibility' and '{}'".format(type(other).__name__))
        return Compressibility(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for **: 'Compressibility' and '{}'".format(type(other).__name__))
        return Compressibility(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for ==: 'Compressibility' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for <: 'Compressibility' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for >: 'Compressibility' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for <=: 'Compressibility' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Compressibility):
            raise TypeError("unsupported operand type(s) for >=: 'Compressibility' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class LengthUnits(Enum):
        """
            LengthUnits enumeration
        """
        
        Meter = 'meter'
        """
            
        """
        
        Mile = 'mile'
        """
            The statute mile was standardised between the British Commonwealth and the United States by an international agreement in 1959, when it was formally redefined with respect to SI units as exactly 1,609.344 metres.
        """
        
        Yard = 'yard'
        """
            The yard (symbol: yd) is an English unit of length in both the British imperial and US customary systems of measurement equalling 3 feet (or 36 inches). Since 1959 the yard has been by international agreement standardized as exactly 0.9144 meter. A distance of 1,760 yards is equal to 1 mile.
        """
        
        Foot = 'foot'
        """
            
        """
        
        UsSurveyFoot = 'us_survey_foot'
        """
            In the United States, the foot was defined as 12 inches, with the inch being defined by the Mendenhall Order of 1893 as 39.37 inches = 1 m. This makes a U.S. survey foot exactly 1200/3937 meters.
        """
        
        Inch = 'inch'
        """
            
        """
        
        Mil = 'mil'
        """
            
        """
        
        NauticalMile = 'nautical_mile'
        """
            
        """
        
        Fathom = 'fathom'
        """
            
        """
        
        Shackle = 'shackle'
        """
            
        """
        
        Microinch = 'microinch'
        """
            
        """
        
        PrinterPoint = 'printer_point'
        """
            
        """
        
        DtpPoint = 'dtp_point'
        """
            
        """
        
        PrinterPica = 'printer_pica'
        """
            
        """
        
        DtpPica = 'dtp_pica'
        """
            
        """
        
        Twip = 'twip'
        """
            
        """
        
        Hand = 'hand'
        """
            
        """
        
        AstronomicalUnit = 'astronomical_unit'
        """
            One Astronomical Unit is the distance from the solar system Star, the sun, to planet Earth.
        """
        
        Parsec = 'parsec'
        """
            A parsec is defined as the distance at which one astronomical unit (AU) subtends an angle of one arcsecond.
        """
        
        LightYear = 'light_year'
        """
            A Light Year (ly) is the distance that light travel during an Earth year, ie 365 days.
        """
        
        SolarRadius = 'solar_radius'
        """
            Solar radius is a ratio unit to the radius of the solar system star, the sun.
        """
        
        Chain = 'chain'
        """
            
        """
        
        Angstrom = 'angstrom'
        """
            Angstrom is a metric unit of length equal to 1e-10 meter
        """
        
        DataMile = 'data_mile'
        """
            In radar-related subjects and in JTIDS, a data mile is a unit of distance equal to 6000 feet (1.8288 kilometres or 0.987 nautical miles).
        """
        
        Nanometer = 'nanometer'
        """
            
        """
        
        Micrometer = 'micrometer'
        """
            
        """
        
        Millimeter = 'millimeter'
        """
            
        """
        
        Centimeter = 'centimeter'
        """
            
        """
        
        Decimeter = 'decimeter'
        """
            
        """
        
        Decameter = 'decameter'
        """
            
        """
        
        Hectometer = 'hectometer'
        """
            
        """
        
        Kilometer = 'kilometer'
        """
            
        """
        
        Megameter = 'megameter'
        """
            
        """
        
        Kilofoot = 'kilofoot'
        """
            
        """
        
        Kiloparsec = 'kiloparsec'
        """
            
        """
        
        Megaparsec = 'megaparsec'
        """
            
        """
        
        KilolightYear = 'kilolight_year'
        """
            
        """
        
        MegalightYear = 'megalight_year'
        """
            
        """
        

class Length:
    """
    Many different units of length have been used around the world. The main units in modern use are U.S. customary units in the United States and the Metric system elsewhere. British Imperial units are still used for some purposes in the United Kingdom and some other countries. The metric system is sub-divided into SI and non-SI units.

    Args:
        value (float): The value.
        from_unit (LengthUnits): The Length unit to create from, The default unit is Meter
    """
    def __init__(self, value: float, from_unit: LengthUnits = LengthUnits.Meter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__meters = None
        
        self.__miles = None
        
        self.__yards = None
        
        self.__feet = None
        
        self.__us_survey_feet = None
        
        self.__inches = None
        
        self.__mils = None
        
        self.__nautical_miles = None
        
        self.__fathoms = None
        
        self.__shackles = None
        
        self.__microinches = None
        
        self.__printer_points = None
        
        self.__dtp_points = None
        
        self.__printer_picas = None
        
        self.__dtp_picas = None
        
        self.__twips = None
        
        self.__hands = None
        
        self.__astronomical_units = None
        
        self.__parsecs = None
        
        self.__light_years = None
        
        self.__solar_radiuses = None
        
        self.__chains = None
        
        self.__angstroms = None
        
        self.__data_miles = None
        
        self.__nanometers = None
        
        self.__micrometers = None
        
        self.__millimeters = None
        
        self.__centimeters = None
        
        self.__decimeters = None
        
        self.__decameters = None
        
        self.__hectometers = None
        
        self.__kilometers = None
        
        self.__megameters = None
        
        self.__kilofeet = None
        
        self.__kiloparsecs = None
        
        self.__megaparsecs = None
        
        self.__kilolight_years = None
        
        self.__megalight_years = None
        

    def __convert_from_base(self, from_unit: LengthUnits) -> float:
        value = self.__value
        
        if from_unit == LengthUnits.Meter:
            return (value)
        
        if from_unit == LengthUnits.Mile:
            return (value / 1609.344)
        
        if from_unit == LengthUnits.Yard:
            return (value / 0.9144)
        
        if from_unit == LengthUnits.Foot:
            return (value / 0.3048)
        
        if from_unit == LengthUnits.UsSurveyFoot:
            return (value * 3937 / 1200)
        
        if from_unit == LengthUnits.Inch:
            return (value / 2.54e-2)
        
        if from_unit == LengthUnits.Mil:
            return (value / 2.54e-5)
        
        if from_unit == LengthUnits.NauticalMile:
            return (value / 1852)
        
        if from_unit == LengthUnits.Fathom:
            return (value / 1.8288)
        
        if from_unit == LengthUnits.Shackle:
            return (value / 27.432)
        
        if from_unit == LengthUnits.Microinch:
            return (value / 2.54e-8)
        
        if from_unit == LengthUnits.PrinterPoint:
            return ((value / 2.54e-2) * 72.27)
        
        if from_unit == LengthUnits.DtpPoint:
            return ((value / 2.54e-2) * 72)
        
        if from_unit == LengthUnits.PrinterPica:
            return (value * 237.106301584)
        
        if from_unit == LengthUnits.DtpPica:
            return (value * 236.220472441)
        
        if from_unit == LengthUnits.Twip:
            return (value * 56692.913385826)
        
        if from_unit == LengthUnits.Hand:
            return (value / 1.016e-1)
        
        if from_unit == LengthUnits.AstronomicalUnit:
            return (value / 1.4959787070e11)
        
        if from_unit == LengthUnits.Parsec:
            return (value / 3.08567758128e16)
        
        if from_unit == LengthUnits.LightYear:
            return (value / 9.46073047258e15)
        
        if from_unit == LengthUnits.SolarRadius:
            return (value / 6.95510000e+08)
        
        if from_unit == LengthUnits.Chain:
            return (value / 20.1168)
        
        if from_unit == LengthUnits.Angstrom:
            return (value / 1e-10)
        
        if from_unit == LengthUnits.DataMile:
            return (value / 1828.8)
        
        if from_unit == LengthUnits.Nanometer:
            return ((value) / 1e-09)
        
        if from_unit == LengthUnits.Micrometer:
            return ((value) / 1e-06)
        
        if from_unit == LengthUnits.Millimeter:
            return ((value) / 0.001)
        
        if from_unit == LengthUnits.Centimeter:
            return ((value) / 0.01)
        
        if from_unit == LengthUnits.Decimeter:
            return ((value) / 0.1)
        
        if from_unit == LengthUnits.Decameter:
            return ((value) / 10.0)
        
        if from_unit == LengthUnits.Hectometer:
            return ((value) / 100.0)
        
        if from_unit == LengthUnits.Kilometer:
            return ((value) / 1000.0)
        
        if from_unit == LengthUnits.Megameter:
            return ((value) / 1000000.0)
        
        if from_unit == LengthUnits.Kilofoot:
            return ((value / 0.3048) / 1000.0)
        
        if from_unit == LengthUnits.Kiloparsec:
            return ((value / 3.08567758128e16) / 1000.0)
        
        if from_unit == LengthUnits.Megaparsec:
            return ((value / 3.08567758128e16) / 1000000.0)
        
        if from_unit == LengthUnits.KilolightYear:
            return ((value / 9.46073047258e15) / 1000.0)
        
        if from_unit == LengthUnits.MegalightYear:
            return ((value / 9.46073047258e15) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LengthUnits) -> float:
        
        if to_unit == LengthUnits.Meter:
            return (value)
        
        if to_unit == LengthUnits.Mile:
            return (value * 1609.344)
        
        if to_unit == LengthUnits.Yard:
            return (value * 0.9144)
        
        if to_unit == LengthUnits.Foot:
            return (value * 0.3048)
        
        if to_unit == LengthUnits.UsSurveyFoot:
            return (value * 1200 / 3937)
        
        if to_unit == LengthUnits.Inch:
            return (value * 2.54e-2)
        
        if to_unit == LengthUnits.Mil:
            return (value * 2.54e-5)
        
        if to_unit == LengthUnits.NauticalMile:
            return (value * 1852)
        
        if to_unit == LengthUnits.Fathom:
            return (value * 1.8288)
        
        if to_unit == LengthUnits.Shackle:
            return (value * 27.432)
        
        if to_unit == LengthUnits.Microinch:
            return (value * 2.54e-8)
        
        if to_unit == LengthUnits.PrinterPoint:
            return ((value / 72.27) * 2.54e-2)
        
        if to_unit == LengthUnits.DtpPoint:
            return ((value / 72) * 2.54e-2)
        
        if to_unit == LengthUnits.PrinterPica:
            return (value / 237.106301584)
        
        if to_unit == LengthUnits.DtpPica:
            return (value / 236.220472441)
        
        if to_unit == LengthUnits.Twip:
            return (value / 56692.913385826)
        
        if to_unit == LengthUnits.Hand:
            return (value * 1.016e-1)
        
        if to_unit == LengthUnits.AstronomicalUnit:
            return (value * 1.4959787070e11)
        
        if to_unit == LengthUnits.Parsec:
            return (value * 3.08567758128e16)
        
        if to_unit == LengthUnits.LightYear:
            return (value * 9.46073047258e15)
        
        if to_unit == LengthUnits.SolarRadius:
            return (value * 6.95510000e+08)
        
        if to_unit == LengthUnits.Chain:
            return (value * 20.1168)
        
        if to_unit == LengthUnits.Angstrom:
            return (value * 1e-10)
        
        if to_unit == LengthUnits.DataMile:
            return (value * 1828.8)
        
        if to_unit == LengthUnits.Nanometer:
            return ((value) * 1e-09)
        
        if to_unit == LengthUnits.Micrometer:
            return ((value) * 1e-06)
        
        if to_unit == LengthUnits.Millimeter:
            return ((value) * 0.001)
        
        if to_unit == LengthUnits.Centimeter:
            return ((value) * 0.01)
        
        if to_unit == LengthUnits.Decimeter:
            return ((value) * 0.1)
        
        if to_unit == LengthUnits.Decameter:
            return ((value) * 10.0)
        
        if to_unit == LengthUnits.Hectometer:
            return ((value) * 100.0)
        
        if to_unit == LengthUnits.Kilometer:
            return ((value) * 1000.0)
        
        if to_unit == LengthUnits.Megameter:
            return ((value) * 1000000.0)
        
        if to_unit == LengthUnits.Kilofoot:
            return ((value * 0.3048) * 1000.0)
        
        if to_unit == LengthUnits.Kiloparsec:
            return ((value * 3.08567758128e16) * 1000.0)
        
        if to_unit == LengthUnits.Megaparsec:
            return ((value * 3.08567758128e16) * 1000000.0)
        
        if to_unit == LengthUnits.KilolightYear:
            return ((value * 9.46073047258e15) * 1000.0)
        
        if to_unit == LengthUnits.MegalightYear:
            return ((value * 9.46073047258e15) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_meters(meters: float):
        """
        Create a new instance of Length from a value in meters.

        

        :param meters: The Length value in meters.
        :type meters: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(meters, LengthUnits.Meter)

    
    @staticmethod
    def from_miles(miles: float):
        """
        Create a new instance of Length from a value in miles.

        The statute mile was standardised between the British Commonwealth and the United States by an international agreement in 1959, when it was formally redefined with respect to SI units as exactly 1,609.344 metres.

        :param meters: The Length value in miles.
        :type miles: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(miles, LengthUnits.Mile)

    
    @staticmethod
    def from_yards(yards: float):
        """
        Create a new instance of Length from a value in yards.

        The yard (symbol: yd) is an English unit of length in both the British imperial and US customary systems of measurement equalling 3 feet (or 36 inches). Since 1959 the yard has been by international agreement standardized as exactly 0.9144 meter. A distance of 1,760 yards is equal to 1 mile.

        :param meters: The Length value in yards.
        :type yards: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(yards, LengthUnits.Yard)

    
    @staticmethod
    def from_feet(feet: float):
        """
        Create a new instance of Length from a value in feet.

        

        :param meters: The Length value in feet.
        :type feet: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(feet, LengthUnits.Foot)

    
    @staticmethod
    def from_us_survey_feet(us_survey_feet: float):
        """
        Create a new instance of Length from a value in us_survey_feet.

        In the United States, the foot was defined as 12 inches, with the inch being defined by the Mendenhall Order of 1893 as 39.37 inches = 1 m. This makes a U.S. survey foot exactly 1200/3937 meters.

        :param meters: The Length value in us_survey_feet.
        :type us_survey_feet: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(us_survey_feet, LengthUnits.UsSurveyFoot)

    
    @staticmethod
    def from_inches(inches: float):
        """
        Create a new instance of Length from a value in inches.

        

        :param meters: The Length value in inches.
        :type inches: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(inches, LengthUnits.Inch)

    
    @staticmethod
    def from_mils(mils: float):
        """
        Create a new instance of Length from a value in mils.

        

        :param meters: The Length value in mils.
        :type mils: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(mils, LengthUnits.Mil)

    
    @staticmethod
    def from_nautical_miles(nautical_miles: float):
        """
        Create a new instance of Length from a value in nautical_miles.

        

        :param meters: The Length value in nautical_miles.
        :type nautical_miles: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(nautical_miles, LengthUnits.NauticalMile)

    
    @staticmethod
    def from_fathoms(fathoms: float):
        """
        Create a new instance of Length from a value in fathoms.

        

        :param meters: The Length value in fathoms.
        :type fathoms: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(fathoms, LengthUnits.Fathom)

    
    @staticmethod
    def from_shackles(shackles: float):
        """
        Create a new instance of Length from a value in shackles.

        

        :param meters: The Length value in shackles.
        :type shackles: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(shackles, LengthUnits.Shackle)

    
    @staticmethod
    def from_microinches(microinches: float):
        """
        Create a new instance of Length from a value in microinches.

        

        :param meters: The Length value in microinches.
        :type microinches: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(microinches, LengthUnits.Microinch)

    
    @staticmethod
    def from_printer_points(printer_points: float):
        """
        Create a new instance of Length from a value in printer_points.

        

        :param meters: The Length value in printer_points.
        :type printer_points: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(printer_points, LengthUnits.PrinterPoint)

    
    @staticmethod
    def from_dtp_points(dtp_points: float):
        """
        Create a new instance of Length from a value in dtp_points.

        

        :param meters: The Length value in dtp_points.
        :type dtp_points: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(dtp_points, LengthUnits.DtpPoint)

    
    @staticmethod
    def from_printer_picas(printer_picas: float):
        """
        Create a new instance of Length from a value in printer_picas.

        

        :param meters: The Length value in printer_picas.
        :type printer_picas: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(printer_picas, LengthUnits.PrinterPica)

    
    @staticmethod
    def from_dtp_picas(dtp_picas: float):
        """
        Create a new instance of Length from a value in dtp_picas.

        

        :param meters: The Length value in dtp_picas.
        :type dtp_picas: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(dtp_picas, LengthUnits.DtpPica)

    
    @staticmethod
    def from_twips(twips: float):
        """
        Create a new instance of Length from a value in twips.

        

        :param meters: The Length value in twips.
        :type twips: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(twips, LengthUnits.Twip)

    
    @staticmethod
    def from_hands(hands: float):
        """
        Create a new instance of Length from a value in hands.

        

        :param meters: The Length value in hands.
        :type hands: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(hands, LengthUnits.Hand)

    
    @staticmethod
    def from_astronomical_units(astronomical_units: float):
        """
        Create a new instance of Length from a value in astronomical_units.

        One Astronomical Unit is the distance from the solar system Star, the sun, to planet Earth.

        :param meters: The Length value in astronomical_units.
        :type astronomical_units: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(astronomical_units, LengthUnits.AstronomicalUnit)

    
    @staticmethod
    def from_parsecs(parsecs: float):
        """
        Create a new instance of Length from a value in parsecs.

        A parsec is defined as the distance at which one astronomical unit (AU) subtends an angle of one arcsecond.

        :param meters: The Length value in parsecs.
        :type parsecs: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(parsecs, LengthUnits.Parsec)

    
    @staticmethod
    def from_light_years(light_years: float):
        """
        Create a new instance of Length from a value in light_years.

        A Light Year (ly) is the distance that light travel during an Earth year, ie 365 days.

        :param meters: The Length value in light_years.
        :type light_years: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(light_years, LengthUnits.LightYear)

    
    @staticmethod
    def from_solar_radiuses(solar_radiuses: float):
        """
        Create a new instance of Length from a value in solar_radiuses.

        Solar radius is a ratio unit to the radius of the solar system star, the sun.

        :param meters: The Length value in solar_radiuses.
        :type solar_radiuses: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(solar_radiuses, LengthUnits.SolarRadius)

    
    @staticmethod
    def from_chains(chains: float):
        """
        Create a new instance of Length from a value in chains.

        

        :param meters: The Length value in chains.
        :type chains: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(chains, LengthUnits.Chain)

    
    @staticmethod
    def from_angstroms(angstroms: float):
        """
        Create a new instance of Length from a value in angstroms.

        Angstrom is a metric unit of length equal to 1e-10 meter

        :param meters: The Length value in angstroms.
        :type angstroms: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(angstroms, LengthUnits.Angstrom)

    
    @staticmethod
    def from_data_miles(data_miles: float):
        """
        Create a new instance of Length from a value in data_miles.

        In radar-related subjects and in JTIDS, a data mile is a unit of distance equal to 6000 feet (1.8288 kilometres or 0.987 nautical miles).

        :param meters: The Length value in data_miles.
        :type data_miles: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(data_miles, LengthUnits.DataMile)

    
    @staticmethod
    def from_nanometers(nanometers: float):
        """
        Create a new instance of Length from a value in nanometers.

        

        :param meters: The Length value in nanometers.
        :type nanometers: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(nanometers, LengthUnits.Nanometer)

    
    @staticmethod
    def from_micrometers(micrometers: float):
        """
        Create a new instance of Length from a value in micrometers.

        

        :param meters: The Length value in micrometers.
        :type micrometers: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(micrometers, LengthUnits.Micrometer)

    
    @staticmethod
    def from_millimeters(millimeters: float):
        """
        Create a new instance of Length from a value in millimeters.

        

        :param meters: The Length value in millimeters.
        :type millimeters: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(millimeters, LengthUnits.Millimeter)

    
    @staticmethod
    def from_centimeters(centimeters: float):
        """
        Create a new instance of Length from a value in centimeters.

        

        :param meters: The Length value in centimeters.
        :type centimeters: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(centimeters, LengthUnits.Centimeter)

    
    @staticmethod
    def from_decimeters(decimeters: float):
        """
        Create a new instance of Length from a value in decimeters.

        

        :param meters: The Length value in decimeters.
        :type decimeters: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(decimeters, LengthUnits.Decimeter)

    
    @staticmethod
    def from_decameters(decameters: float):
        """
        Create a new instance of Length from a value in decameters.

        

        :param meters: The Length value in decameters.
        :type decameters: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(decameters, LengthUnits.Decameter)

    
    @staticmethod
    def from_hectometers(hectometers: float):
        """
        Create a new instance of Length from a value in hectometers.

        

        :param meters: The Length value in hectometers.
        :type hectometers: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(hectometers, LengthUnits.Hectometer)

    
    @staticmethod
    def from_kilometers(kilometers: float):
        """
        Create a new instance of Length from a value in kilometers.

        

        :param meters: The Length value in kilometers.
        :type kilometers: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(kilometers, LengthUnits.Kilometer)

    
    @staticmethod
    def from_megameters(megameters: float):
        """
        Create a new instance of Length from a value in megameters.

        

        :param meters: The Length value in megameters.
        :type megameters: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(megameters, LengthUnits.Megameter)

    
    @staticmethod
    def from_kilofeet(kilofeet: float):
        """
        Create a new instance of Length from a value in kilofeet.

        

        :param meters: The Length value in kilofeet.
        :type kilofeet: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(kilofeet, LengthUnits.Kilofoot)

    
    @staticmethod
    def from_kiloparsecs(kiloparsecs: float):
        """
        Create a new instance of Length from a value in kiloparsecs.

        

        :param meters: The Length value in kiloparsecs.
        :type kiloparsecs: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(kiloparsecs, LengthUnits.Kiloparsec)

    
    @staticmethod
    def from_megaparsecs(megaparsecs: float):
        """
        Create a new instance of Length from a value in megaparsecs.

        

        :param meters: The Length value in megaparsecs.
        :type megaparsecs: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(megaparsecs, LengthUnits.Megaparsec)

    
    @staticmethod
    def from_kilolight_years(kilolight_years: float):
        """
        Create a new instance of Length from a value in kilolight_years.

        

        :param meters: The Length value in kilolight_years.
        :type kilolight_years: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(kilolight_years, LengthUnits.KilolightYear)

    
    @staticmethod
    def from_megalight_years(megalight_years: float):
        """
        Create a new instance of Length from a value in megalight_years.

        

        :param meters: The Length value in megalight_years.
        :type megalight_years: float
        :return: A new instance of Length.
        :rtype: Length
        """
        return Length(megalight_years, LengthUnits.MegalightYear)

    
    @property
    def meters(self) -> float:
        """
        
        """
        if self.__meters != None:
            return self.__meters
        self.__meters = self.__convert_from_base(LengthUnits.Meter)
        return self.__meters

    
    @property
    def miles(self) -> float:
        """
        The statute mile was standardised between the British Commonwealth and the United States by an international agreement in 1959, when it was formally redefined with respect to SI units as exactly 1,609.344 metres.
        """
        if self.__miles != None:
            return self.__miles
        self.__miles = self.__convert_from_base(LengthUnits.Mile)
        return self.__miles

    
    @property
    def yards(self) -> float:
        """
        The yard (symbol: yd) is an English unit of length in both the British imperial and US customary systems of measurement equalling 3 feet (or 36 inches). Since 1959 the yard has been by international agreement standardized as exactly 0.9144 meter. A distance of 1,760 yards is equal to 1 mile.
        """
        if self.__yards != None:
            return self.__yards
        self.__yards = self.__convert_from_base(LengthUnits.Yard)
        return self.__yards

    
    @property
    def feet(self) -> float:
        """
        
        """
        if self.__feet != None:
            return self.__feet
        self.__feet = self.__convert_from_base(LengthUnits.Foot)
        return self.__feet

    
    @property
    def us_survey_feet(self) -> float:
        """
        In the United States, the foot was defined as 12 inches, with the inch being defined by the Mendenhall Order of 1893 as 39.37 inches = 1 m. This makes a U.S. survey foot exactly 1200/3937 meters.
        """
        if self.__us_survey_feet != None:
            return self.__us_survey_feet
        self.__us_survey_feet = self.__convert_from_base(LengthUnits.UsSurveyFoot)
        return self.__us_survey_feet

    
    @property
    def inches(self) -> float:
        """
        
        """
        if self.__inches != None:
            return self.__inches
        self.__inches = self.__convert_from_base(LengthUnits.Inch)
        return self.__inches

    
    @property
    def mils(self) -> float:
        """
        
        """
        if self.__mils != None:
            return self.__mils
        self.__mils = self.__convert_from_base(LengthUnits.Mil)
        return self.__mils

    
    @property
    def nautical_miles(self) -> float:
        """
        
        """
        if self.__nautical_miles != None:
            return self.__nautical_miles
        self.__nautical_miles = self.__convert_from_base(LengthUnits.NauticalMile)
        return self.__nautical_miles

    
    @property
    def fathoms(self) -> float:
        """
        
        """
        if self.__fathoms != None:
            return self.__fathoms
        self.__fathoms = self.__convert_from_base(LengthUnits.Fathom)
        return self.__fathoms

    
    @property
    def shackles(self) -> float:
        """
        
        """
        if self.__shackles != None:
            return self.__shackles
        self.__shackles = self.__convert_from_base(LengthUnits.Shackle)
        return self.__shackles

    
    @property
    def microinches(self) -> float:
        """
        
        """
        if self.__microinches != None:
            return self.__microinches
        self.__microinches = self.__convert_from_base(LengthUnits.Microinch)
        return self.__microinches

    
    @property
    def printer_points(self) -> float:
        """
        
        """
        if self.__printer_points != None:
            return self.__printer_points
        self.__printer_points = self.__convert_from_base(LengthUnits.PrinterPoint)
        return self.__printer_points

    
    @property
    def dtp_points(self) -> float:
        """
        
        """
        if self.__dtp_points != None:
            return self.__dtp_points
        self.__dtp_points = self.__convert_from_base(LengthUnits.DtpPoint)
        return self.__dtp_points

    
    @property
    def printer_picas(self) -> float:
        """
        
        """
        if self.__printer_picas != None:
            return self.__printer_picas
        self.__printer_picas = self.__convert_from_base(LengthUnits.PrinterPica)
        return self.__printer_picas

    
    @property
    def dtp_picas(self) -> float:
        """
        
        """
        if self.__dtp_picas != None:
            return self.__dtp_picas
        self.__dtp_picas = self.__convert_from_base(LengthUnits.DtpPica)
        return self.__dtp_picas

    
    @property
    def twips(self) -> float:
        """
        
        """
        if self.__twips != None:
            return self.__twips
        self.__twips = self.__convert_from_base(LengthUnits.Twip)
        return self.__twips

    
    @property
    def hands(self) -> float:
        """
        
        """
        if self.__hands != None:
            return self.__hands
        self.__hands = self.__convert_from_base(LengthUnits.Hand)
        return self.__hands

    
    @property
    def astronomical_units(self) -> float:
        """
        One Astronomical Unit is the distance from the solar system Star, the sun, to planet Earth.
        """
        if self.__astronomical_units != None:
            return self.__astronomical_units
        self.__astronomical_units = self.__convert_from_base(LengthUnits.AstronomicalUnit)
        return self.__astronomical_units

    
    @property
    def parsecs(self) -> float:
        """
        A parsec is defined as the distance at which one astronomical unit (AU) subtends an angle of one arcsecond.
        """
        if self.__parsecs != None:
            return self.__parsecs
        self.__parsecs = self.__convert_from_base(LengthUnits.Parsec)
        return self.__parsecs

    
    @property
    def light_years(self) -> float:
        """
        A Light Year (ly) is the distance that light travel during an Earth year, ie 365 days.
        """
        if self.__light_years != None:
            return self.__light_years
        self.__light_years = self.__convert_from_base(LengthUnits.LightYear)
        return self.__light_years

    
    @property
    def solar_radiuses(self) -> float:
        """
        Solar radius is a ratio unit to the radius of the solar system star, the sun.
        """
        if self.__solar_radiuses != None:
            return self.__solar_radiuses
        self.__solar_radiuses = self.__convert_from_base(LengthUnits.SolarRadius)
        return self.__solar_radiuses

    
    @property
    def chains(self) -> float:
        """
        
        """
        if self.__chains != None:
            return self.__chains
        self.__chains = self.__convert_from_base(LengthUnits.Chain)
        return self.__chains

    
    @property
    def angstroms(self) -> float:
        """
        Angstrom is a metric unit of length equal to 1e-10 meter
        """
        if self.__angstroms != None:
            return self.__angstroms
        self.__angstroms = self.__convert_from_base(LengthUnits.Angstrom)
        return self.__angstroms

    
    @property
    def data_miles(self) -> float:
        """
        In radar-related subjects and in JTIDS, a data mile is a unit of distance equal to 6000 feet (1.8288 kilometres or 0.987 nautical miles).
        """
        if self.__data_miles != None:
            return self.__data_miles
        self.__data_miles = self.__convert_from_base(LengthUnits.DataMile)
        return self.__data_miles

    
    @property
    def nanometers(self) -> float:
        """
        
        """
        if self.__nanometers != None:
            return self.__nanometers
        self.__nanometers = self.__convert_from_base(LengthUnits.Nanometer)
        return self.__nanometers

    
    @property
    def micrometers(self) -> float:
        """
        
        """
        if self.__micrometers != None:
            return self.__micrometers
        self.__micrometers = self.__convert_from_base(LengthUnits.Micrometer)
        return self.__micrometers

    
    @property
    def millimeters(self) -> float:
        """
        
        """
        if self.__millimeters != None:
            return self.__millimeters
        self.__millimeters = self.__convert_from_base(LengthUnits.Millimeter)
        return self.__millimeters

    
    @property
    def centimeters(self) -> float:
        """
        
        """
        if self.__centimeters != None:
            return self.__centimeters
        self.__centimeters = self.__convert_from_base(LengthUnits.Centimeter)
        return self.__centimeters

    
    @property
    def decimeters(self) -> float:
        """
        
        """
        if self.__decimeters != None:
            return self.__decimeters
        self.__decimeters = self.__convert_from_base(LengthUnits.Decimeter)
        return self.__decimeters

    
    @property
    def decameters(self) -> float:
        """
        
        """
        if self.__decameters != None:
            return self.__decameters
        self.__decameters = self.__convert_from_base(LengthUnits.Decameter)
        return self.__decameters

    
    @property
    def hectometers(self) -> float:
        """
        
        """
        if self.__hectometers != None:
            return self.__hectometers
        self.__hectometers = self.__convert_from_base(LengthUnits.Hectometer)
        return self.__hectometers

    
    @property
    def kilometers(self) -> float:
        """
        
        """
        if self.__kilometers != None:
            return self.__kilometers
        self.__kilometers = self.__convert_from_base(LengthUnits.Kilometer)
        return self.__kilometers

    
    @property
    def megameters(self) -> float:
        """
        
        """
        if self.__megameters != None:
            return self.__megameters
        self.__megameters = self.__convert_from_base(LengthUnits.Megameter)
        return self.__megameters

    
    @property
    def kilofeet(self) -> float:
        """
        
        """
        if self.__kilofeet != None:
            return self.__kilofeet
        self.__kilofeet = self.__convert_from_base(LengthUnits.Kilofoot)
        return self.__kilofeet

    
    @property
    def kiloparsecs(self) -> float:
        """
        
        """
        if self.__kiloparsecs != None:
            return self.__kiloparsecs
        self.__kiloparsecs = self.__convert_from_base(LengthUnits.Kiloparsec)
        return self.__kiloparsecs

    
    @property
    def megaparsecs(self) -> float:
        """
        
        """
        if self.__megaparsecs != None:
            return self.__megaparsecs
        self.__megaparsecs = self.__convert_from_base(LengthUnits.Megaparsec)
        return self.__megaparsecs

    
    @property
    def kilolight_years(self) -> float:
        """
        
        """
        if self.__kilolight_years != None:
            return self.__kilolight_years
        self.__kilolight_years = self.__convert_from_base(LengthUnits.KilolightYear)
        return self.__kilolight_years

    
    @property
    def megalight_years(self) -> float:
        """
        
        """
        if self.__megalight_years != None:
            return self.__megalight_years
        self.__megalight_years = self.__convert_from_base(LengthUnits.MegalightYear)
        return self.__megalight_years

    
    def to_string(self, unit: LengthUnits = LengthUnits.Meter) -> string:
        """
        Format the Length to string.
        Note! the default format for Length is Meter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == LengthUnits.Meter:
            return f"""{self.meters} m"""
        
        if unit == LengthUnits.Mile:
            return f"""{self.miles} mi"""
        
        if unit == LengthUnits.Yard:
            return f"""{self.yards} yd"""
        
        if unit == LengthUnits.Foot:
            return f"""{self.feet} ft"""
        
        if unit == LengthUnits.UsSurveyFoot:
            return f"""{self.us_survey_feet} ftUS"""
        
        if unit == LengthUnits.Inch:
            return f"""{self.inches} in"""
        
        if unit == LengthUnits.Mil:
            return f"""{self.mils} mil"""
        
        if unit == LengthUnits.NauticalMile:
            return f"""{self.nautical_miles} NM"""
        
        if unit == LengthUnits.Fathom:
            return f"""{self.fathoms} fathom"""
        
        if unit == LengthUnits.Shackle:
            return f"""{self.shackles} shackle"""
        
        if unit == LengthUnits.Microinch:
            return f"""{self.microinches} µin"""
        
        if unit == LengthUnits.PrinterPoint:
            return f"""{self.printer_points} pt"""
        
        if unit == LengthUnits.DtpPoint:
            return f"""{self.dtp_points} pt"""
        
        if unit == LengthUnits.PrinterPica:
            return f"""{self.printer_picas} pica"""
        
        if unit == LengthUnits.DtpPica:
            return f"""{self.dtp_picas} pica"""
        
        if unit == LengthUnits.Twip:
            return f"""{self.twips} twip"""
        
        if unit == LengthUnits.Hand:
            return f"""{self.hands} h"""
        
        if unit == LengthUnits.AstronomicalUnit:
            return f"""{self.astronomical_units} au"""
        
        if unit == LengthUnits.Parsec:
            return f"""{self.parsecs} pc"""
        
        if unit == LengthUnits.LightYear:
            return f"""{self.light_years} ly"""
        
        if unit == LengthUnits.SolarRadius:
            return f"""{self.solar_radiuses} R⊙"""
        
        if unit == LengthUnits.Chain:
            return f"""{self.chains} ch"""
        
        if unit == LengthUnits.Angstrom:
            return f"""{self.angstroms} Å"""
        
        if unit == LengthUnits.DataMile:
            return f"""{self.data_miles} DM"""
        
        if unit == LengthUnits.Nanometer:
            return f"""{self.nanometers} """
        
        if unit == LengthUnits.Micrometer:
            return f"""{self.micrometers} """
        
        if unit == LengthUnits.Millimeter:
            return f"""{self.millimeters} """
        
        if unit == LengthUnits.Centimeter:
            return f"""{self.centimeters} """
        
        if unit == LengthUnits.Decimeter:
            return f"""{self.decimeters} """
        
        if unit == LengthUnits.Decameter:
            return f"""{self.decameters} """
        
        if unit == LengthUnits.Hectometer:
            return f"""{self.hectometers} """
        
        if unit == LengthUnits.Kilometer:
            return f"""{self.kilometers} """
        
        if unit == LengthUnits.Megameter:
            return f"""{self.megameters} """
        
        if unit == LengthUnits.Kilofoot:
            return f"""{self.kilofeet} """
        
        if unit == LengthUnits.Kiloparsec:
            return f"""{self.kiloparsecs} """
        
        if unit == LengthUnits.Megaparsec:
            return f"""{self.megaparsecs} """
        
        if unit == LengthUnits.KilolightYear:
            return f"""{self.kilolight_years} """
        
        if unit == LengthUnits.MegalightYear:
            return f"""{self.megalight_years} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: LengthUnits = LengthUnits.Meter) -> string:
        """
        Get Length unit abbreviation.
        Note! the default abbreviation for Length is Meter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LengthUnits.Meter:
            return """m"""
        
        if unit_abbreviation == LengthUnits.Mile:
            return """mi"""
        
        if unit_abbreviation == LengthUnits.Yard:
            return """yd"""
        
        if unit_abbreviation == LengthUnits.Foot:
            return """ft"""
        
        if unit_abbreviation == LengthUnits.UsSurveyFoot:
            return """ftUS"""
        
        if unit_abbreviation == LengthUnits.Inch:
            return """in"""
        
        if unit_abbreviation == LengthUnits.Mil:
            return """mil"""
        
        if unit_abbreviation == LengthUnits.NauticalMile:
            return """NM"""
        
        if unit_abbreviation == LengthUnits.Fathom:
            return """fathom"""
        
        if unit_abbreviation == LengthUnits.Shackle:
            return """shackle"""
        
        if unit_abbreviation == LengthUnits.Microinch:
            return """µin"""
        
        if unit_abbreviation == LengthUnits.PrinterPoint:
            return """pt"""
        
        if unit_abbreviation == LengthUnits.DtpPoint:
            return """pt"""
        
        if unit_abbreviation == LengthUnits.PrinterPica:
            return """pica"""
        
        if unit_abbreviation == LengthUnits.DtpPica:
            return """pica"""
        
        if unit_abbreviation == LengthUnits.Twip:
            return """twip"""
        
        if unit_abbreviation == LengthUnits.Hand:
            return """h"""
        
        if unit_abbreviation == LengthUnits.AstronomicalUnit:
            return """au"""
        
        if unit_abbreviation == LengthUnits.Parsec:
            return """pc"""
        
        if unit_abbreviation == LengthUnits.LightYear:
            return """ly"""
        
        if unit_abbreviation == LengthUnits.SolarRadius:
            return """R⊙"""
        
        if unit_abbreviation == LengthUnits.Chain:
            return """ch"""
        
        if unit_abbreviation == LengthUnits.Angstrom:
            return """Å"""
        
        if unit_abbreviation == LengthUnits.DataMile:
            return """DM"""
        
        if unit_abbreviation == LengthUnits.Nanometer:
            return """"""
        
        if unit_abbreviation == LengthUnits.Micrometer:
            return """"""
        
        if unit_abbreviation == LengthUnits.Millimeter:
            return """"""
        
        if unit_abbreviation == LengthUnits.Centimeter:
            return """"""
        
        if unit_abbreviation == LengthUnits.Decimeter:
            return """"""
        
        if unit_abbreviation == LengthUnits.Decameter:
            return """"""
        
        if unit_abbreviation == LengthUnits.Hectometer:
            return """"""
        
        if unit_abbreviation == LengthUnits.Kilometer:
            return """"""
        
        if unit_abbreviation == LengthUnits.Megameter:
            return """"""
        
        if unit_abbreviation == LengthUnits.Kilofoot:
            return """"""
        
        if unit_abbreviation == LengthUnits.Kiloparsec:
            return """"""
        
        if unit_abbreviation == LengthUnits.Megaparsec:
            return """"""
        
        if unit_abbreviation == LengthUnits.KilolightYear:
            return """"""
        
        if unit_abbreviation == LengthUnits.MegalightYear:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for +: 'Length' and '{}'".format(type(other).__name__))
        return Length(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for *: 'Length' and '{}'".format(type(other).__name__))
        return Length(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for -: 'Length' and '{}'".format(type(other).__name__))
        return Length(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for /: 'Length' and '{}'".format(type(other).__name__))
        return Length(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for %: 'Length' and '{}'".format(type(other).__name__))
        return Length(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for **: 'Length' and '{}'".format(type(other).__name__))
        return Length(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for ==: 'Length' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for <: 'Length' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for >: 'Length' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for <=: 'Length' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Length):
            raise TypeError("unsupported operand type(s) for >=: 'Length' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class InformationUnits(Enum):
        """
            InformationUnits enumeration
        """
        
        Byte = 'byte'
        """
            
        """
        
        Bit = 'bit'
        """
            
        """
        
        Kilobyte = 'kilobyte'
        """
            
        """
        
        Megabyte = 'megabyte'
        """
            
        """
        
        Gigabyte = 'gigabyte'
        """
            
        """
        
        Terabyte = 'terabyte'
        """
            
        """
        
        Petabyte = 'petabyte'
        """
            
        """
        
        Exabyte = 'exabyte'
        """
            
        """
        
        Kilobit = 'kilobit'
        """
            
        """
        
        Megabit = 'megabit'
        """
            
        """
        
        Gigabit = 'gigabit'
        """
            
        """
        
        Terabit = 'terabit'
        """
            
        """
        
        Petabit = 'petabit'
        """
            
        """
        
        Exabit = 'exabit'
        """
            
        """
        

class Information:
    """
    In computing and telecommunications, a unit of information is the capacity of some standard data storage system or communication channel, used to measure the capacities of other systems and channels. In information theory, units of information are also used to measure the information contents or entropy of random variables.

    Args:
        value (float): The value.
        from_unit (InformationUnits): The Information unit to create from, The default unit is Bit
    """
    def __init__(self, value: float, from_unit: InformationUnits = InformationUnits.Bit):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__bytes = None
        
        self.__bits = None
        
        self.__kilobytes = None
        
        self.__megabytes = None
        
        self.__gigabytes = None
        
        self.__terabytes = None
        
        self.__petabytes = None
        
        self.__exabytes = None
        
        self.__kilobits = None
        
        self.__megabits = None
        
        self.__gigabits = None
        
        self.__terabits = None
        
        self.__petabits = None
        
        self.__exabits = None
        

    def __convert_from_base(self, from_unit: InformationUnits) -> float:
        value = self.__value
        
        if from_unit == InformationUnits.Byte:
            return (value / 8)
        
        if from_unit == InformationUnits.Bit:
            return (value)
        
        if from_unit == InformationUnits.Kilobyte:
            return ((value / 8) / 1000.0)
        
        if from_unit == InformationUnits.Megabyte:
            return ((value / 8) / 1000000.0)
        
        if from_unit == InformationUnits.Gigabyte:
            return ((value / 8) / 1000000000.0)
        
        if from_unit == InformationUnits.Terabyte:
            return ((value / 8) / 1000000000000.0)
        
        if from_unit == InformationUnits.Petabyte:
            return ((value / 8) / 1000000000000000.0)
        
        if from_unit == InformationUnits.Exabyte:
            return ((value / 8) / 1e+18)
        
        if from_unit == InformationUnits.Kilobit:
            return ((value) / 1000.0)
        
        if from_unit == InformationUnits.Megabit:
            return ((value) / 1000000.0)
        
        if from_unit == InformationUnits.Gigabit:
            return ((value) / 1000000000.0)
        
        if from_unit == InformationUnits.Terabit:
            return ((value) / 1000000000000.0)
        
        if from_unit == InformationUnits.Petabit:
            return ((value) / 1000000000000000.0)
        
        if from_unit == InformationUnits.Exabit:
            return ((value) / 1e+18)
        
        return None


    def __convert_to_base(self, value: float, to_unit: InformationUnits) -> float:
        
        if to_unit == InformationUnits.Byte:
            return (value * 8)
        
        if to_unit == InformationUnits.Bit:
            return (value)
        
        if to_unit == InformationUnits.Kilobyte:
            return ((value * 8) * 1000.0)
        
        if to_unit == InformationUnits.Megabyte:
            return ((value * 8) * 1000000.0)
        
        if to_unit == InformationUnits.Gigabyte:
            return ((value * 8) * 1000000000.0)
        
        if to_unit == InformationUnits.Terabyte:
            return ((value * 8) * 1000000000000.0)
        
        if to_unit == InformationUnits.Petabyte:
            return ((value * 8) * 1000000000000000.0)
        
        if to_unit == InformationUnits.Exabyte:
            return ((value * 8) * 1e+18)
        
        if to_unit == InformationUnits.Kilobit:
            return ((value) * 1000.0)
        
        if to_unit == InformationUnits.Megabit:
            return ((value) * 1000000.0)
        
        if to_unit == InformationUnits.Gigabit:
            return ((value) * 1000000000.0)
        
        if to_unit == InformationUnits.Terabit:
            return ((value) * 1000000000000.0)
        
        if to_unit == InformationUnits.Petabit:
            return ((value) * 1000000000000000.0)
        
        if to_unit == InformationUnits.Exabit:
            return ((value) * 1e+18)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_bytes(bytes: float):
        """
        Create a new instance of Information from a value in bytes.

        

        :param meters: The Information value in bytes.
        :type bytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(bytes, InformationUnits.Byte)

    
    @staticmethod
    def from_bits(bits: float):
        """
        Create a new instance of Information from a value in bits.

        

        :param meters: The Information value in bits.
        :type bits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(bits, InformationUnits.Bit)

    
    @staticmethod
    def from_kilobytes(kilobytes: float):
        """
        Create a new instance of Information from a value in kilobytes.

        

        :param meters: The Information value in kilobytes.
        :type kilobytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(kilobytes, InformationUnits.Kilobyte)

    
    @staticmethod
    def from_megabytes(megabytes: float):
        """
        Create a new instance of Information from a value in megabytes.

        

        :param meters: The Information value in megabytes.
        :type megabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(megabytes, InformationUnits.Megabyte)

    
    @staticmethod
    def from_gigabytes(gigabytes: float):
        """
        Create a new instance of Information from a value in gigabytes.

        

        :param meters: The Information value in gigabytes.
        :type gigabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(gigabytes, InformationUnits.Gigabyte)

    
    @staticmethod
    def from_terabytes(terabytes: float):
        """
        Create a new instance of Information from a value in terabytes.

        

        :param meters: The Information value in terabytes.
        :type terabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(terabytes, InformationUnits.Terabyte)

    
    @staticmethod
    def from_petabytes(petabytes: float):
        """
        Create a new instance of Information from a value in petabytes.

        

        :param meters: The Information value in petabytes.
        :type petabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(petabytes, InformationUnits.Petabyte)

    
    @staticmethod
    def from_exabytes(exabytes: float):
        """
        Create a new instance of Information from a value in exabytes.

        

        :param meters: The Information value in exabytes.
        :type exabytes: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(exabytes, InformationUnits.Exabyte)

    
    @staticmethod
    def from_kilobits(kilobits: float):
        """
        Create a new instance of Information from a value in kilobits.

        

        :param meters: The Information value in kilobits.
        :type kilobits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(kilobits, InformationUnits.Kilobit)

    
    @staticmethod
    def from_megabits(megabits: float):
        """
        Create a new instance of Information from a value in megabits.

        

        :param meters: The Information value in megabits.
        :type megabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(megabits, InformationUnits.Megabit)

    
    @staticmethod
    def from_gigabits(gigabits: float):
        """
        Create a new instance of Information from a value in gigabits.

        

        :param meters: The Information value in gigabits.
        :type gigabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(gigabits, InformationUnits.Gigabit)

    
    @staticmethod
    def from_terabits(terabits: float):
        """
        Create a new instance of Information from a value in terabits.

        

        :param meters: The Information value in terabits.
        :type terabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(terabits, InformationUnits.Terabit)

    
    @staticmethod
    def from_petabits(petabits: float):
        """
        Create a new instance of Information from a value in petabits.

        

        :param meters: The Information value in petabits.
        :type petabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(petabits, InformationUnits.Petabit)

    
    @staticmethod
    def from_exabits(exabits: float):
        """
        Create a new instance of Information from a value in exabits.

        

        :param meters: The Information value in exabits.
        :type exabits: float
        :return: A new instance of Information.
        :rtype: Information
        """
        return Information(exabits, InformationUnits.Exabit)

    
    @property
    def bytes(self) -> float:
        """
        
        """
        if self.__bytes != None:
            return self.__bytes
        self.__bytes = self.__convert_from_base(InformationUnits.Byte)
        return self.__bytes

    
    @property
    def bits(self) -> float:
        """
        
        """
        if self.__bits != None:
            return self.__bits
        self.__bits = self.__convert_from_base(InformationUnits.Bit)
        return self.__bits

    
    @property
    def kilobytes(self) -> float:
        """
        
        """
        if self.__kilobytes != None:
            return self.__kilobytes
        self.__kilobytes = self.__convert_from_base(InformationUnits.Kilobyte)
        return self.__kilobytes

    
    @property
    def megabytes(self) -> float:
        """
        
        """
        if self.__megabytes != None:
            return self.__megabytes
        self.__megabytes = self.__convert_from_base(InformationUnits.Megabyte)
        return self.__megabytes

    
    @property
    def gigabytes(self) -> float:
        """
        
        """
        if self.__gigabytes != None:
            return self.__gigabytes
        self.__gigabytes = self.__convert_from_base(InformationUnits.Gigabyte)
        return self.__gigabytes

    
    @property
    def terabytes(self) -> float:
        """
        
        """
        if self.__terabytes != None:
            return self.__terabytes
        self.__terabytes = self.__convert_from_base(InformationUnits.Terabyte)
        return self.__terabytes

    
    @property
    def petabytes(self) -> float:
        """
        
        """
        if self.__petabytes != None:
            return self.__petabytes
        self.__petabytes = self.__convert_from_base(InformationUnits.Petabyte)
        return self.__petabytes

    
    @property
    def exabytes(self) -> float:
        """
        
        """
        if self.__exabytes != None:
            return self.__exabytes
        self.__exabytes = self.__convert_from_base(InformationUnits.Exabyte)
        return self.__exabytes

    
    @property
    def kilobits(self) -> float:
        """
        
        """
        if self.__kilobits != None:
            return self.__kilobits
        self.__kilobits = self.__convert_from_base(InformationUnits.Kilobit)
        return self.__kilobits

    
    @property
    def megabits(self) -> float:
        """
        
        """
        if self.__megabits != None:
            return self.__megabits
        self.__megabits = self.__convert_from_base(InformationUnits.Megabit)
        return self.__megabits

    
    @property
    def gigabits(self) -> float:
        """
        
        """
        if self.__gigabits != None:
            return self.__gigabits
        self.__gigabits = self.__convert_from_base(InformationUnits.Gigabit)
        return self.__gigabits

    
    @property
    def terabits(self) -> float:
        """
        
        """
        if self.__terabits != None:
            return self.__terabits
        self.__terabits = self.__convert_from_base(InformationUnits.Terabit)
        return self.__terabits

    
    @property
    def petabits(self) -> float:
        """
        
        """
        if self.__petabits != None:
            return self.__petabits
        self.__petabits = self.__convert_from_base(InformationUnits.Petabit)
        return self.__petabits

    
    @property
    def exabits(self) -> float:
        """
        
        """
        if self.__exabits != None:
            return self.__exabits
        self.__exabits = self.__convert_from_base(InformationUnits.Exabit)
        return self.__exabits

    
    def to_string(self, unit: InformationUnits = InformationUnits.Bit) -> string:
        """
        Format the Information to string.
        Note! the default format for Information is Bit.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == InformationUnits.Byte:
            return f"""{self.bytes} B"""
        
        if unit == InformationUnits.Bit:
            return f"""{self.bits} b"""
        
        if unit == InformationUnits.Kilobyte:
            return f"""{self.kilobytes} """
        
        if unit == InformationUnits.Megabyte:
            return f"""{self.megabytes} """
        
        if unit == InformationUnits.Gigabyte:
            return f"""{self.gigabytes} """
        
        if unit == InformationUnits.Terabyte:
            return f"""{self.terabytes} """
        
        if unit == InformationUnits.Petabyte:
            return f"""{self.petabytes} """
        
        if unit == InformationUnits.Exabyte:
            return f"""{self.exabytes} """
        
        if unit == InformationUnits.Kilobit:
            return f"""{self.kilobits} """
        
        if unit == InformationUnits.Megabit:
            return f"""{self.megabits} """
        
        if unit == InformationUnits.Gigabit:
            return f"""{self.gigabits} """
        
        if unit == InformationUnits.Terabit:
            return f"""{self.terabits} """
        
        if unit == InformationUnits.Petabit:
            return f"""{self.petabits} """
        
        if unit == InformationUnits.Exabit:
            return f"""{self.exabits} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: InformationUnits = InformationUnits.Bit) -> string:
        """
        Get Information unit abbreviation.
        Note! the default abbreviation for Information is Bit.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == InformationUnits.Byte:
            return """B"""
        
        if unit_abbreviation == InformationUnits.Bit:
            return """b"""
        
        if unit_abbreviation == InformationUnits.Kilobyte:
            return """"""
        
        if unit_abbreviation == InformationUnits.Megabyte:
            return """"""
        
        if unit_abbreviation == InformationUnits.Gigabyte:
            return """"""
        
        if unit_abbreviation == InformationUnits.Terabyte:
            return """"""
        
        if unit_abbreviation == InformationUnits.Petabyte:
            return """"""
        
        if unit_abbreviation == InformationUnits.Exabyte:
            return """"""
        
        if unit_abbreviation == InformationUnits.Kilobit:
            return """"""
        
        if unit_abbreviation == InformationUnits.Megabit:
            return """"""
        
        if unit_abbreviation == InformationUnits.Gigabit:
            return """"""
        
        if unit_abbreviation == InformationUnits.Terabit:
            return """"""
        
        if unit_abbreviation == InformationUnits.Petabit:
            return """"""
        
        if unit_abbreviation == InformationUnits.Exabit:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for +: 'Information' and '{}'".format(type(other).__name__))
        return Information(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for *: 'Information' and '{}'".format(type(other).__name__))
        return Information(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for -: 'Information' and '{}'".format(type(other).__name__))
        return Information(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for /: 'Information' and '{}'".format(type(other).__name__))
        return Information(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for %: 'Information' and '{}'".format(type(other).__name__))
        return Information(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for **: 'Information' and '{}'".format(type(other).__name__))
        return Information(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for ==: 'Information' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for <: 'Information' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for >: 'Information' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for <=: 'Information' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Information):
            raise TypeError("unsupported operand type(s) for >=: 'Information' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class AmplitudeRatioUnits(Enum):
        """
            AmplitudeRatioUnits enumeration
        """
        
        DecibelVolt = 'decibel_volt'
        """
            
        """
        
        DecibelMicrovolt = 'decibel_microvolt'
        """
            
        """
        
        DecibelMillivolt = 'decibel_millivolt'
        """
            
        """
        
        DecibelUnloaded = 'decibel_unloaded'
        """
            
        """
        

class AmplitudeRatio:
    """
    The strength of a signal expressed in decibels (dB) relative to one volt RMS.

    Args:
        value (float): The value.
        from_unit (AmplitudeRatioUnits): The AmplitudeRatio unit to create from, The default unit is DecibelVolt
    """
    def __init__(self, value: float, from_unit: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__decibel_volts = None
        
        self.__decibel_microvolts = None
        
        self.__decibel_millivolts = None
        
        self.__decibels_unloaded = None
        

    def __convert_from_base(self, from_unit: AmplitudeRatioUnits) -> float:
        value = self.__value
        
        if from_unit == AmplitudeRatioUnits.DecibelVolt:
            return (value)
        
        if from_unit == AmplitudeRatioUnits.DecibelMicrovolt:
            return (value + 120)
        
        if from_unit == AmplitudeRatioUnits.DecibelMillivolt:
            return (value + 60)
        
        if from_unit == AmplitudeRatioUnits.DecibelUnloaded:
            return (value + 2.218487499)
        
        return None


    def __convert_to_base(self, value: float, to_unit: AmplitudeRatioUnits) -> float:
        
        if to_unit == AmplitudeRatioUnits.DecibelVolt:
            return (value)
        
        if to_unit == AmplitudeRatioUnits.DecibelMicrovolt:
            return (value - 120)
        
        if to_unit == AmplitudeRatioUnits.DecibelMillivolt:
            return (value - 60)
        
        if to_unit == AmplitudeRatioUnits.DecibelUnloaded:
            return (value - 2.218487499)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_decibel_volts(decibel_volts: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibel_volts.

        

        :param meters: The AmplitudeRatio value in decibel_volts.
        :type decibel_volts: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibel_volts, AmplitudeRatioUnits.DecibelVolt)

    
    @staticmethod
    def from_decibel_microvolts(decibel_microvolts: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibel_microvolts.

        

        :param meters: The AmplitudeRatio value in decibel_microvolts.
        :type decibel_microvolts: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibel_microvolts, AmplitudeRatioUnits.DecibelMicrovolt)

    
    @staticmethod
    def from_decibel_millivolts(decibel_millivolts: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibel_millivolts.

        

        :param meters: The AmplitudeRatio value in decibel_millivolts.
        :type decibel_millivolts: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibel_millivolts, AmplitudeRatioUnits.DecibelMillivolt)

    
    @staticmethod
    def from_decibels_unloaded(decibels_unloaded: float):
        """
        Create a new instance of AmplitudeRatio from a value in decibels_unloaded.

        

        :param meters: The AmplitudeRatio value in decibels_unloaded.
        :type decibels_unloaded: float
        :return: A new instance of AmplitudeRatio.
        :rtype: AmplitudeRatio
        """
        return AmplitudeRatio(decibels_unloaded, AmplitudeRatioUnits.DecibelUnloaded)

    
    @property
    def decibel_volts(self) -> float:
        """
        
        """
        if self.__decibel_volts != None:
            return self.__decibel_volts
        self.__decibel_volts = self.__convert_from_base(AmplitudeRatioUnits.DecibelVolt)
        return self.__decibel_volts

    
    @property
    def decibel_microvolts(self) -> float:
        """
        
        """
        if self.__decibel_microvolts != None:
            return self.__decibel_microvolts
        self.__decibel_microvolts = self.__convert_from_base(AmplitudeRatioUnits.DecibelMicrovolt)
        return self.__decibel_microvolts

    
    @property
    def decibel_millivolts(self) -> float:
        """
        
        """
        if self.__decibel_millivolts != None:
            return self.__decibel_millivolts
        self.__decibel_millivolts = self.__convert_from_base(AmplitudeRatioUnits.DecibelMillivolt)
        return self.__decibel_millivolts

    
    @property
    def decibels_unloaded(self) -> float:
        """
        
        """
        if self.__decibels_unloaded != None:
            return self.__decibels_unloaded
        self.__decibels_unloaded = self.__convert_from_base(AmplitudeRatioUnits.DecibelUnloaded)
        return self.__decibels_unloaded

    
    def to_string(self, unit: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt) -> string:
        """
        Format the AmplitudeRatio to string.
        Note! the default format for AmplitudeRatio is DecibelVolt.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == AmplitudeRatioUnits.DecibelVolt:
            return f"""{self.decibel_volts} dBV"""
        
        if unit == AmplitudeRatioUnits.DecibelMicrovolt:
            return f"""{self.decibel_microvolts} dBµV"""
        
        if unit == AmplitudeRatioUnits.DecibelMillivolt:
            return f"""{self.decibel_millivolts} dBmV"""
        
        if unit == AmplitudeRatioUnits.DecibelUnloaded:
            return f"""{self.decibels_unloaded} dBu"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: AmplitudeRatioUnits = AmplitudeRatioUnits.DecibelVolt) -> string:
        """
        Get AmplitudeRatio unit abbreviation.
        Note! the default abbreviation for AmplitudeRatio is DecibelVolt.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelVolt:
            return """dBV"""
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelMicrovolt:
            return """dBµV"""
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelMillivolt:
            return """dBmV"""
        
        if unit_abbreviation == AmplitudeRatioUnits.DecibelUnloaded:
            return """dBu"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for +: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return AmplitudeRatio(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for *: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return AmplitudeRatio(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for -: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return AmplitudeRatio(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for /: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return AmplitudeRatio(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for %: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return AmplitudeRatio(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for **: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return AmplitudeRatio(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for ==: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for <: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for >: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for <=: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, AmplitudeRatio):
            raise TypeError("unsupported operand type(s) for >=: 'AmplitudeRatio' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class TemperatureDeltaUnits(Enum):
        """
            TemperatureDeltaUnits enumeration
        """
        
        Kelvin = 'kelvin'
        """
            
        """
        
        DegreeCelsius = 'degree_celsius'
        """
            
        """
        
        DegreeDelisle = 'degree_delisle'
        """
            
        """
        
        DegreeFahrenheit = 'degree_fahrenheit'
        """
            
        """
        
        DegreeNewton = 'degree_newton'
        """
            
        """
        
        DegreeRankine = 'degree_rankine'
        """
            
        """
        
        DegreeReaumur = 'degree_reaumur'
        """
            
        """
        
        DegreeRoemer = 'degree_roemer'
        """
            
        """
        
        MillidegreeCelsius = 'millidegree_celsius'
        """
            
        """
        

class TemperatureDelta:
    """
    Difference between two temperatures. The conversions are different than for Temperature.

    Args:
        value (float): The value.
        from_unit (TemperatureDeltaUnits): The TemperatureDelta unit to create from, The default unit is Kelvin
    """
    def __init__(self, value: float, from_unit: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__kelvins = None
        
        self.__degrees_celsius = None
        
        self.__degrees_delisle = None
        
        self.__degrees_fahrenheit = None
        
        self.__degrees_newton = None
        
        self.__degrees_rankine = None
        
        self.__degrees_reaumur = None
        
        self.__degrees_roemer = None
        
        self.__millidegrees_celsius = None
        

    def __convert_from_base(self, from_unit: TemperatureDeltaUnits) -> float:
        value = self.__value
        
        if from_unit == TemperatureDeltaUnits.Kelvin:
            return (value)
        
        if from_unit == TemperatureDeltaUnits.DegreeCelsius:
            return (value)
        
        if from_unit == TemperatureDeltaUnits.DegreeDelisle:
            return (value * -3 / 2)
        
        if from_unit == TemperatureDeltaUnits.DegreeFahrenheit:
            return (value * 9 / 5)
        
        if from_unit == TemperatureDeltaUnits.DegreeNewton:
            return (value * 33 / 100)
        
        if from_unit == TemperatureDeltaUnits.DegreeRankine:
            return (value * 9 / 5)
        
        if from_unit == TemperatureDeltaUnits.DegreeReaumur:
            return (value * 4 / 5)
        
        if from_unit == TemperatureDeltaUnits.DegreeRoemer:
            return (value * 21 / 40)
        
        if from_unit == TemperatureDeltaUnits.MillidegreeCelsius:
            return ((value) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TemperatureDeltaUnits) -> float:
        
        if to_unit == TemperatureDeltaUnits.Kelvin:
            return (value)
        
        if to_unit == TemperatureDeltaUnits.DegreeCelsius:
            return (value)
        
        if to_unit == TemperatureDeltaUnits.DegreeDelisle:
            return (value * -2 / 3)
        
        if to_unit == TemperatureDeltaUnits.DegreeFahrenheit:
            return (value * 5 / 9)
        
        if to_unit == TemperatureDeltaUnits.DegreeNewton:
            return (value * 100 / 33)
        
        if to_unit == TemperatureDeltaUnits.DegreeRankine:
            return (value * 5 / 9)
        
        if to_unit == TemperatureDeltaUnits.DegreeReaumur:
            return (value * 5 / 4)
        
        if to_unit == TemperatureDeltaUnits.DegreeRoemer:
            return (value * 40 / 21)
        
        if to_unit == TemperatureDeltaUnits.MillidegreeCelsius:
            return ((value) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_kelvins(kelvins: float):
        """
        Create a new instance of TemperatureDelta from a value in kelvins.

        

        :param meters: The TemperatureDelta value in kelvins.
        :type kelvins: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(kelvins, TemperatureDeltaUnits.Kelvin)

    
    @staticmethod
    def from_degrees_celsius(degrees_celsius: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_celsius.

        

        :param meters: The TemperatureDelta value in degrees_celsius.
        :type degrees_celsius: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_celsius, TemperatureDeltaUnits.DegreeCelsius)

    
    @staticmethod
    def from_degrees_delisle(degrees_delisle: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_delisle.

        

        :param meters: The TemperatureDelta value in degrees_delisle.
        :type degrees_delisle: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_delisle, TemperatureDeltaUnits.DegreeDelisle)

    
    @staticmethod
    def from_degrees_fahrenheit(degrees_fahrenheit: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_fahrenheit.

        

        :param meters: The TemperatureDelta value in degrees_fahrenheit.
        :type degrees_fahrenheit: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_fahrenheit, TemperatureDeltaUnits.DegreeFahrenheit)

    
    @staticmethod
    def from_degrees_newton(degrees_newton: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_newton.

        

        :param meters: The TemperatureDelta value in degrees_newton.
        :type degrees_newton: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_newton, TemperatureDeltaUnits.DegreeNewton)

    
    @staticmethod
    def from_degrees_rankine(degrees_rankine: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_rankine.

        

        :param meters: The TemperatureDelta value in degrees_rankine.
        :type degrees_rankine: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_rankine, TemperatureDeltaUnits.DegreeRankine)

    
    @staticmethod
    def from_degrees_reaumur(degrees_reaumur: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_reaumur.

        

        :param meters: The TemperatureDelta value in degrees_reaumur.
        :type degrees_reaumur: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_reaumur, TemperatureDeltaUnits.DegreeReaumur)

    
    @staticmethod
    def from_degrees_roemer(degrees_roemer: float):
        """
        Create a new instance of TemperatureDelta from a value in degrees_roemer.

        

        :param meters: The TemperatureDelta value in degrees_roemer.
        :type degrees_roemer: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(degrees_roemer, TemperatureDeltaUnits.DegreeRoemer)

    
    @staticmethod
    def from_millidegrees_celsius(millidegrees_celsius: float):
        """
        Create a new instance of TemperatureDelta from a value in millidegrees_celsius.

        

        :param meters: The TemperatureDelta value in millidegrees_celsius.
        :type millidegrees_celsius: float
        :return: A new instance of TemperatureDelta.
        :rtype: TemperatureDelta
        """
        return TemperatureDelta(millidegrees_celsius, TemperatureDeltaUnits.MillidegreeCelsius)

    
    @property
    def kelvins(self) -> float:
        """
        
        """
        if self.__kelvins != None:
            return self.__kelvins
        self.__kelvins = self.__convert_from_base(TemperatureDeltaUnits.Kelvin)
        return self.__kelvins

    
    @property
    def degrees_celsius(self) -> float:
        """
        
        """
        if self.__degrees_celsius != None:
            return self.__degrees_celsius
        self.__degrees_celsius = self.__convert_from_base(TemperatureDeltaUnits.DegreeCelsius)
        return self.__degrees_celsius

    
    @property
    def degrees_delisle(self) -> float:
        """
        
        """
        if self.__degrees_delisle != None:
            return self.__degrees_delisle
        self.__degrees_delisle = self.__convert_from_base(TemperatureDeltaUnits.DegreeDelisle)
        return self.__degrees_delisle

    
    @property
    def degrees_fahrenheit(self) -> float:
        """
        
        """
        if self.__degrees_fahrenheit != None:
            return self.__degrees_fahrenheit
        self.__degrees_fahrenheit = self.__convert_from_base(TemperatureDeltaUnits.DegreeFahrenheit)
        return self.__degrees_fahrenheit

    
    @property
    def degrees_newton(self) -> float:
        """
        
        """
        if self.__degrees_newton != None:
            return self.__degrees_newton
        self.__degrees_newton = self.__convert_from_base(TemperatureDeltaUnits.DegreeNewton)
        return self.__degrees_newton

    
    @property
    def degrees_rankine(self) -> float:
        """
        
        """
        if self.__degrees_rankine != None:
            return self.__degrees_rankine
        self.__degrees_rankine = self.__convert_from_base(TemperatureDeltaUnits.DegreeRankine)
        return self.__degrees_rankine

    
    @property
    def degrees_reaumur(self) -> float:
        """
        
        """
        if self.__degrees_reaumur != None:
            return self.__degrees_reaumur
        self.__degrees_reaumur = self.__convert_from_base(TemperatureDeltaUnits.DegreeReaumur)
        return self.__degrees_reaumur

    
    @property
    def degrees_roemer(self) -> float:
        """
        
        """
        if self.__degrees_roemer != None:
            return self.__degrees_roemer
        self.__degrees_roemer = self.__convert_from_base(TemperatureDeltaUnits.DegreeRoemer)
        return self.__degrees_roemer

    
    @property
    def millidegrees_celsius(self) -> float:
        """
        
        """
        if self.__millidegrees_celsius != None:
            return self.__millidegrees_celsius
        self.__millidegrees_celsius = self.__convert_from_base(TemperatureDeltaUnits.MillidegreeCelsius)
        return self.__millidegrees_celsius

    
    def to_string(self, unit: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin) -> string:
        """
        Format the TemperatureDelta to string.
        Note! the default format for TemperatureDelta is Kelvin.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == TemperatureDeltaUnits.Kelvin:
            return f"""{self.kelvins} ∆K"""
        
        if unit == TemperatureDeltaUnits.DegreeCelsius:
            return f"""{self.degrees_celsius} ∆°C"""
        
        if unit == TemperatureDeltaUnits.DegreeDelisle:
            return f"""{self.degrees_delisle} ∆°De"""
        
        if unit == TemperatureDeltaUnits.DegreeFahrenheit:
            return f"""{self.degrees_fahrenheit} ∆°F"""
        
        if unit == TemperatureDeltaUnits.DegreeNewton:
            return f"""{self.degrees_newton} ∆°N"""
        
        if unit == TemperatureDeltaUnits.DegreeRankine:
            return f"""{self.degrees_rankine} ∆°R"""
        
        if unit == TemperatureDeltaUnits.DegreeReaumur:
            return f"""{self.degrees_reaumur} ∆°Ré"""
        
        if unit == TemperatureDeltaUnits.DegreeRoemer:
            return f"""{self.degrees_roemer} ∆°Rø"""
        
        if unit == TemperatureDeltaUnits.MillidegreeCelsius:
            return f"""{self.millidegrees_celsius} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: TemperatureDeltaUnits = TemperatureDeltaUnits.Kelvin) -> string:
        """
        Get TemperatureDelta unit abbreviation.
        Note! the default abbreviation for TemperatureDelta is Kelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TemperatureDeltaUnits.Kelvin:
            return """∆K"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeCelsius:
            return """∆°C"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeDelisle:
            return """∆°De"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeFahrenheit:
            return """∆°F"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeNewton:
            return """∆°N"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeRankine:
            return """∆°R"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeReaumur:
            return """∆°Ré"""
        
        if unit_abbreviation == TemperatureDeltaUnits.DegreeRoemer:
            return """∆°Rø"""
        
        if unit_abbreviation == TemperatureDeltaUnits.MillidegreeCelsius:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for +: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return TemperatureDelta(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for *: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return TemperatureDelta(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for -: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return TemperatureDelta(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for /: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return TemperatureDelta(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for %: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return TemperatureDelta(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for **: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return TemperatureDelta(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for ==: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for <: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for >: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for <=: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, TemperatureDelta):
            raise TypeError("unsupported operand type(s) for >=: 'TemperatureDelta' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class VolumetricHeatCapacityUnits(Enum):
        """
            VolumetricHeatCapacityUnits enumeration
        """
        
        JoulePerCubicMeterKelvin = 'joule_per_cubic_meter_kelvin'
        """
            
        """
        
        JoulePerCubicMeterDegreeCelsius = 'joule_per_cubic_meter_degree_celsius'
        """
            
        """
        
        CaloriePerCubicCentimeterDegreeCelsius = 'calorie_per_cubic_centimeter_degree_celsius'
        """
            
        """
        
        BtuPerCubicFootDegreeFahrenheit = 'btu_per_cubic_foot_degree_fahrenheit'
        """
            
        """
        
        KilojoulePerCubicMeterKelvin = 'kilojoule_per_cubic_meter_kelvin'
        """
            
        """
        
        MegajoulePerCubicMeterKelvin = 'megajoule_per_cubic_meter_kelvin'
        """
            
        """
        
        KilojoulePerCubicMeterDegreeCelsius = 'kilojoule_per_cubic_meter_degree_celsius'
        """
            
        """
        
        MegajoulePerCubicMeterDegreeCelsius = 'megajoule_per_cubic_meter_degree_celsius'
        """
            
        """
        
        KilocaloriePerCubicCentimeterDegreeCelsius = 'kilocalorie_per_cubic_centimeter_degree_celsius'
        """
            
        """
        

class VolumetricHeatCapacity:
    """
    The volumetric heat capacity is the amount of energy that must be added, in the form of heat, to one unit of volume of the material in order to cause an increase of one unit in its temperature.

    Args:
        value (float): The value.
        from_unit (VolumetricHeatCapacityUnits): The VolumetricHeatCapacity unit to create from, The default unit is JoulePerCubicMeterKelvin
    """
    def __init__(self, value: float, from_unit: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_cubic_meter_kelvin = None
        
        self.__joules_per_cubic_meter_degree_celsius = None
        
        self.__calories_per_cubic_centimeter_degree_celsius = None
        
        self.__btus_per_cubic_foot_degree_fahrenheit = None
        
        self.__kilojoules_per_cubic_meter_kelvin = None
        
        self.__megajoules_per_cubic_meter_kelvin = None
        
        self.__kilojoules_per_cubic_meter_degree_celsius = None
        
        self.__megajoules_per_cubic_meter_degree_celsius = None
        
        self.__kilocalories_per_cubic_centimeter_degree_celsius = None
        

    def __convert_from_base(self, from_unit: VolumetricHeatCapacityUnits) -> float:
        value = self.__value
        
        if from_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return (value)
        
        if from_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return (value)
        
        if from_unit == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return (value * 2.388459e-7)
        
        if from_unit == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return (value * 1.4910660e-5)
        
        if from_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return ((value) / 1000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return ((value) / 1000000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return ((value) / 1000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return ((value) / 1000000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return ((value * 2.388459e-7) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: VolumetricHeatCapacityUnits) -> float:
        
        if to_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return (value)
        
        if to_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return (value)
        
        if to_unit == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return (value / 2.388459e-7)
        
        if to_unit == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return (value / 1.4910660e-5)
        
        if to_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return ((value) * 1000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return ((value) * 1000000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return ((value) * 1000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return ((value) * 1000000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return ((value / 2.388459e-7) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_joules_per_cubic_meter_kelvin(joules_per_cubic_meter_kelvin: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in joules_per_cubic_meter_kelvin.

        

        :param meters: The VolumetricHeatCapacity value in joules_per_cubic_meter_kelvin.
        :type joules_per_cubic_meter_kelvin: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(joules_per_cubic_meter_kelvin, VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin)

    
    @staticmethod
    def from_joules_per_cubic_meter_degree_celsius(joules_per_cubic_meter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in joules_per_cubic_meter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in joules_per_cubic_meter_degree_celsius.
        :type joules_per_cubic_meter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(joules_per_cubic_meter_degree_celsius, VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius)

    
    @staticmethod
    def from_calories_per_cubic_centimeter_degree_celsius(calories_per_cubic_centimeter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in calories_per_cubic_centimeter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in calories_per_cubic_centimeter_degree_celsius.
        :type calories_per_cubic_centimeter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(calories_per_cubic_centimeter_degree_celsius, VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius)

    
    @staticmethod
    def from_btus_per_cubic_foot_degree_fahrenheit(btus_per_cubic_foot_degree_fahrenheit: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in btus_per_cubic_foot_degree_fahrenheit.

        

        :param meters: The VolumetricHeatCapacity value in btus_per_cubic_foot_degree_fahrenheit.
        :type btus_per_cubic_foot_degree_fahrenheit: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(btus_per_cubic_foot_degree_fahrenheit, VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit)

    
    @staticmethod
    def from_kilojoules_per_cubic_meter_kelvin(kilojoules_per_cubic_meter_kelvin: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in kilojoules_per_cubic_meter_kelvin.

        

        :param meters: The VolumetricHeatCapacity value in kilojoules_per_cubic_meter_kelvin.
        :type kilojoules_per_cubic_meter_kelvin: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(kilojoules_per_cubic_meter_kelvin, VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin)

    
    @staticmethod
    def from_megajoules_per_cubic_meter_kelvin(megajoules_per_cubic_meter_kelvin: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in megajoules_per_cubic_meter_kelvin.

        

        :param meters: The VolumetricHeatCapacity value in megajoules_per_cubic_meter_kelvin.
        :type megajoules_per_cubic_meter_kelvin: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(megajoules_per_cubic_meter_kelvin, VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin)

    
    @staticmethod
    def from_kilojoules_per_cubic_meter_degree_celsius(kilojoules_per_cubic_meter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in kilojoules_per_cubic_meter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in kilojoules_per_cubic_meter_degree_celsius.
        :type kilojoules_per_cubic_meter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(kilojoules_per_cubic_meter_degree_celsius, VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius)

    
    @staticmethod
    def from_megajoules_per_cubic_meter_degree_celsius(megajoules_per_cubic_meter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in megajoules_per_cubic_meter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in megajoules_per_cubic_meter_degree_celsius.
        :type megajoules_per_cubic_meter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(megajoules_per_cubic_meter_degree_celsius, VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius)

    
    @staticmethod
    def from_kilocalories_per_cubic_centimeter_degree_celsius(kilocalories_per_cubic_centimeter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in kilocalories_per_cubic_centimeter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in kilocalories_per_cubic_centimeter_degree_celsius.
        :type kilocalories_per_cubic_centimeter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(kilocalories_per_cubic_centimeter_degree_celsius, VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius)

    
    @property
    def joules_per_cubic_meter_kelvin(self) -> float:
        """
        
        """
        if self.__joules_per_cubic_meter_kelvin != None:
            return self.__joules_per_cubic_meter_kelvin
        self.__joules_per_cubic_meter_kelvin = self.__convert_from_base(VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin)
        return self.__joules_per_cubic_meter_kelvin

    
    @property
    def joules_per_cubic_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__joules_per_cubic_meter_degree_celsius != None:
            return self.__joules_per_cubic_meter_degree_celsius
        self.__joules_per_cubic_meter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius)
        return self.__joules_per_cubic_meter_degree_celsius

    
    @property
    def calories_per_cubic_centimeter_degree_celsius(self) -> float:
        """
        
        """
        if self.__calories_per_cubic_centimeter_degree_celsius != None:
            return self.__calories_per_cubic_centimeter_degree_celsius
        self.__calories_per_cubic_centimeter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius)
        return self.__calories_per_cubic_centimeter_degree_celsius

    
    @property
    def btus_per_cubic_foot_degree_fahrenheit(self) -> float:
        """
        
        """
        if self.__btus_per_cubic_foot_degree_fahrenheit != None:
            return self.__btus_per_cubic_foot_degree_fahrenheit
        self.__btus_per_cubic_foot_degree_fahrenheit = self.__convert_from_base(VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit)
        return self.__btus_per_cubic_foot_degree_fahrenheit

    
    @property
    def kilojoules_per_cubic_meter_kelvin(self) -> float:
        """
        
        """
        if self.__kilojoules_per_cubic_meter_kelvin != None:
            return self.__kilojoules_per_cubic_meter_kelvin
        self.__kilojoules_per_cubic_meter_kelvin = self.__convert_from_base(VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin)
        return self.__kilojoules_per_cubic_meter_kelvin

    
    @property
    def megajoules_per_cubic_meter_kelvin(self) -> float:
        """
        
        """
        if self.__megajoules_per_cubic_meter_kelvin != None:
            return self.__megajoules_per_cubic_meter_kelvin
        self.__megajoules_per_cubic_meter_kelvin = self.__convert_from_base(VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin)
        return self.__megajoules_per_cubic_meter_kelvin

    
    @property
    def kilojoules_per_cubic_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilojoules_per_cubic_meter_degree_celsius != None:
            return self.__kilojoules_per_cubic_meter_degree_celsius
        self.__kilojoules_per_cubic_meter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius)
        return self.__kilojoules_per_cubic_meter_degree_celsius

    
    @property
    def megajoules_per_cubic_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__megajoules_per_cubic_meter_degree_celsius != None:
            return self.__megajoules_per_cubic_meter_degree_celsius
        self.__megajoules_per_cubic_meter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius)
        return self.__megajoules_per_cubic_meter_degree_celsius

    
    @property
    def kilocalories_per_cubic_centimeter_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilocalories_per_cubic_centimeter_degree_celsius != None:
            return self.__kilocalories_per_cubic_centimeter_degree_celsius
        self.__kilocalories_per_cubic_centimeter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius)
        return self.__kilocalories_per_cubic_centimeter_degree_celsius

    
    def to_string(self, unit: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin) -> string:
        """
        Format the VolumetricHeatCapacity to string.
        Note! the default format for VolumetricHeatCapacity is JoulePerCubicMeterKelvin.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return f"""{self.joules_per_cubic_meter_kelvin} J/m³·K"""
        
        if unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return f"""{self.joules_per_cubic_meter_degree_celsius} J/m³·°C"""
        
        if unit == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return f"""{self.calories_per_cubic_centimeter_degree_celsius} cal/cm³·°C"""
        
        if unit == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return f"""{self.btus_per_cubic_foot_degree_fahrenheit} BTU/ft³·°F"""
        
        if unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return f"""{self.kilojoules_per_cubic_meter_kelvin} """
        
        if unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return f"""{self.megajoules_per_cubic_meter_kelvin} """
        
        if unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return f"""{self.kilojoules_per_cubic_meter_degree_celsius} """
        
        if unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return f"""{self.megajoules_per_cubic_meter_degree_celsius} """
        
        if unit == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return f"""{self.kilocalories_per_cubic_centimeter_degree_celsius} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin) -> string:
        """
        Get VolumetricHeatCapacity unit abbreviation.
        Note! the default abbreviation for VolumetricHeatCapacity is JoulePerCubicMeterKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return """J/m³·K"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return """J/m³·°C"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return """cal/cm³·°C"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return """BTU/ft³·°F"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return """"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return """"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return """"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return """"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for +: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return VolumetricHeatCapacity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for *: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return VolumetricHeatCapacity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for -: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return VolumetricHeatCapacity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for /: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return VolumetricHeatCapacity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for %: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return VolumetricHeatCapacity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for **: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return VolumetricHeatCapacity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for ==: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for <: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for >: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for <=: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, VolumetricHeatCapacity):
            raise TypeError("unsupported operand type(s) for >=: 'VolumetricHeatCapacity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value